/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2020-2022 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/***********************************************************************/
/*                                                                     */
/*  FILE        :Main.c or Main.cpp                                    */
/*  DATE        :                                                      */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
#include "r_cg_macrodriver.h"
#include "Config_TAU0_0.h"
#include "falling_block_puzzle.h"

unsigned char field[FIELD_HEIGHT][FIELD_WIDTH];
unsigned char screen[FIELD_HEIGHT][FIELD_WIDTH];
unsigned char blockX;
unsigned char blockY;
unsigned char block_type;
unsigned char block_angle;
unsigned char line_fill;
unsigned char key_sw;

extern unsigned char F_20ms;
extern unsigned char random;

const unsigned char block_shapes[BLOCK_MAX][BLOCK_ANGLE_MAX][BLOCK_HEIGHT][BLOCK_WIDTH] = {
	//STICK_BLOCK
	{
		//BLOCK_ANGLE_0
		{
			0,1,0,0,
			0,1,0,0,
			0,1,0,0,
			0,1,0,0,
		},
		//BLOCK_ANGLE_90
		{
			0,0,0,0,
			0,0,0,0,
			1,1,1,1,
			0,0,0,0,
		},
		//BLOCK_ANGLE_180
		{
			0,0,1,0,
			0,0,1,0,
			0,0,1,0,
			0,0,1,0,
		},
		//BLOCK_ANGLE_270
		{
			0,0,0,0,
			1,1,1,1,
			0,0,0,0,
			0,0,0,0,
		},
	},
	//SQUARE_BLOCK
	{
		//BLOCK_ANGLE_0
		{
			0,0,0,0,
			0,1,1,0,
			0,1,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_90
		{
			0,0,0,0,
			0,1,1,0,
			0,1,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_180
		{
			0,0,0,0,
			0,1,1,0,
			0,1,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_270
		{
			0,0,0,0,
			0,1,1,0,
			0,1,1,0,
			0,0,0,0,
		},
	},
	//MOUNT_BLOCK
	{
		//BLOCK_ANGLE_0
		{
			0,0,0,0,
			1,1,1,0,
			0,1,0,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_90
		{
			0,0,0,0,
			0,1,0,0,
			0,1,1,0,
			0,1,0,0,
		},
		//BLOCK_ANGLE_180
		{
			0,0,0,0,
			0,0,1,0,
			0,1,1,1,
			0,0,0,0,
		},
		//BLOCK_ANGLE_270
		{
			0,0,1,0,
			0,1,1,0,
			0,0,1,0,
			0,0,0,0,
		},
	},
	//LWORD1_BLOCK
	{
		//BLOCK_ANGLE_0
		{
			0,0,1,0,
			0,0,1,0,
			0,1,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_90
		{
			0,0,0,0,
			1,1,1,0,
			0,0,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_180
		{
			0,0,0,0,
			0,1,1,0,
			0,1,0,0,
			0,1,0,0,
		},
		//BLOCK_ANGLE_270
		{
			0,0,0,0,
			0,1,0,0,
			0,1,1,1,
			0,0,0,0,
		},
	},
	//LWORD2_BLOCK
	{
		//BLOCK_ANGLE_0
		{
			0,1,0,0,
			0,1,0,0,
			0,1,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_90
		{
			0,0,0,0,
			0,0,1,0,
			1,1,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_180
		{
			0,0,0,0,
			0,1,1,0,
			0,0,1,0,
			0,0,1,0,
		},
		//BLOCK_ANGLE_270
		{
			0,0,0,0,
			0,1,1,1,
			0,1,0,0,
			0,0,0,0,
		},
	},
	//TWIST1_BLOCK
	{
		//BLOCK_ANGLE_0
		{
			0,0,0,0,
			0,1,1,0,
			1,1,0,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_90
		{
			0,1,0,0,
			0,1,1,0,
			0,0,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_180
		{
			0,0,0,0,
			0,1,1,0,
			1,1,0,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_270
		{
			0,0,0,0,
			0,1,0,0,
			0,1,1,0,
			0,0,1,0,
		},
	},
	//TWIST2_BLOCK
	{
		//BLOCK_ANGLE_0
		{
			0,0,0,0,
			1,1,0,0,
			0,1,1,0,
			0,0,0,0,
		},
		//BLOCK_ANGLE_90
		{
			0,0,0,0,
			0,0,1,0,
			0,1,1,0,
			0,1,0,0,
		},
		//BLOCK_ANGLE_180
		{
			0,0,0,0,
			0,1,1,0,
			0,0,1,1,
			0,0,0,0,
		},
		//BLOCK_ANGLE_270
		{
			0,0,1,0,
			0,1,1,0,
			0,1,0,0,
			0,0,0,0,
		},
	},
};

void Field_Clear(void);
void Screen_Clear(void);
void main(void);

void main(void)
{
	R_Config_TAU0_0_Start();
	EI();

	Block_Reset();
	Field_Clear();
	Field_Copy();

	while(1U)
	{
		if(P5 == 0x0F)
		{
			key_sw = 0;	//Processing when the switch is released
		}
		HALT();
	}
}

/***********************************************************************************************************************
* Function Name: Field_Clear
* Description  : Function to clear a field
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Field_Clear(void)
{
	for(int i = 0; i < FIELD_HEIGHT; i++)
	{
		for(int j = 0; j < FIELD_WIDTH; j++)
		{
			if(i == (FIELD_HEIGHT - 1) | j == 0 | j == (FIELD_WIDTH - 1))
			{
				field[i][j] = 1;
			}
			else
			{
				field[i][j] = 0;
			}
		}
	}
}

/***********************************************************************************************************************
* Function Name: Screen_Clear
* Description  : Function to clear screen
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Screen_Clear(void)
{
	for(int i = 0; i < FIELD_HEIGHT; i++)
	{
		for(int j = 0; j < FIELD_WIDTH; j++)
		{
			if(i == (FIELD_HEIGHT - 1) | j == 0 | j == (FIELD_WIDTH - 1))
			{
				screen[i][j] = 1;
			}
			else
			{
				screen[i][j] = 0;
			}
		}
	}
}

/***********************************************************************************************************************
* Function Name: Field_Copy
* Description  : Function to copy a field to the screen
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Field_Copy(void)
{
	for(int i = 0; i < FIELD_HEIGHT; i++)
	{
		for(int j = 0; j < FIELD_WIDTH; j++)
		{
			screen[i][j] |= field[i][j];
		}
	}
}

/***********************************************************************************************************************
* Function Name: Screen_Update
* Description  : Function to draw a block on the screen
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Screen_Update(void)
{
	Screen_Clear();
	for(int i = 0; i < BLOCK_HEIGHT; i++)
	{
		for(int j = 0; j < BLOCK_WIDTH; j++)
		{
			screen[blockY + i][blockX + j] |= block_shapes[block_type][block_angle][i][j];
		}
	}
}

/***********************************************************************************************************************
* Function Name: Block_Hit
* Description  : Function to determine if a block has hit a wall
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
unsigned char Block_Hit(unsigned char _blockX, unsigned char _blockY, unsigned char _block_type, unsigned char _block_angle)
{
	for(int i = 0; i < BLOCK_HEIGHT; i++)
	{
		for(int j = 0; j < BLOCK_WIDTH; j++)
		{
			if(block_shapes[_block_type][_block_angle][i][j] && field[_blockY + i][_blockX + j])
			{
			return true;
			}
		}
	}
	return false;
}

/***********************************************************************************************************************
* Function Name: Sw_Process
* Description  : Function to perform processing based on the switch pressed
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Sw_Process(void)
{
	if(key_sw == 0)
	{
		key_sw = P5;
		switch(key_sw)
		{
			case ROTAITION:
				if(!Block_Hit(blockX, blockY, block_type, (block_angle + 1) % BLOCK_ANGLE_MAX))
				{
					block_angle = (block_angle + 1) % BLOCK_ANGLE_MAX;
				}
				break;
			case RIGHT:
				if(!Block_Hit(blockX + 1, blockY, block_type, block_angle))
				{
					blockX++;
				}
				break;
			case LEFT:
				if(!Block_Hit(blockX - 1, blockY, block_type, block_angle))
				{
					blockX--;
				}
				break;
			case UNDER:
				if(!Block_Hit(blockX, blockY + 1, block_type, block_angle))
				{
					blockY++;
				}
				break;
			default:
				break;
		}
	}
}


/***********************************************************************************************************************
* Function Name: Block_Reset
* Description  : Block Initialization
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Block_Reset(void)
{
	blockX = 3;
	blockY = 0;

	//Generation of random numbers
	TRNGSCR0 = TRNGSCR0 | 0x08;
        TRNGSCR0 = TRNGSCR0 | 0x04;
        while((TRNGSCR0 & 0x80) == 0x00)
	{
        	NOP();
        }
        block_type = (TRNGSDR ^ TRNGSDR ^ TRNGSDR ^ TRNGSDR) % 7;	//Select the shape of the block from the generated random numbers
        TRNGSCR0 = TRNGSCR0 & 0xF7;

	TRNGSCR0 = TRNGSCR0 | 0x08;
        TRNGSCR0 = TRNGSCR0 | 0x04;
        while((TRNGSCR0 & 0x80) == 0x00)
	{
        	NOP();
        }
        block_angle = (TRNGSDR ^ TRNGSDR ^ TRNGSDR ^ TRNGSDR) % 4;	//Select the angle of the block from the generated random numbers
        TRNGSCR0 = TRNGSCR0 & 0xF7;
}

/***********************************************************************************************************************
* Function Name: Line_Filled_Check
* Description  : Function to check if a row of blocks are aligned
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Line_Filled_Check(void)
{
	for(int i = 0; i < FIELD_HEIGHT - 1; i++)
	{
		line_fill = true;
		for(int j = 1; j < FIELD_WIDTH - 1; j++)
		{
			if(!field[i][j])
			{
				line_fill = false;
			}
		}
		if(line_fill)
		{
			for(int j = i; 0 < j; j--)
			{
				for(int k = 0; k < FIELD_WIDTH; k++)
				{
				field[j][k] = field[j - 1][k];
				}
			}
		}
	}
}
