/***********************************************************************************************************************
* File Name        : matrix.c
* Component Version: 1.0.0
* Device(s)        : R7F100GLGxFA
* Description      :
***********************************************************************************************************************/
 /***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "matrix.h"
#include "falling_block_puzzle.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
unsigned char matrix_digit;
unsigned char matrix_data1[5] = {0x00,0x00,0x00,0x00,0x00};
unsigned char matrix_data1_2[5] = {0x00,0x00,0x00,0x00,0x00};
unsigned char matrix_data2[5] = {0x00,0x00,0x00,0x00,0x00};
unsigned char matrix_data3[5] = {0x00,0x00,0x00,0x00,0x00};

extern unsigned char field[FIELD_HEIGHT][FIELD_WIDTH];
extern unsigned char screen[FIELD_HEIGHT][FIELD_WIDTH];

const unsigned char matrix_digit_table[5] = {0x04,0x08,0x10,0x20,0x40};

/***********************************************************************************************************************
* Function Name: Matrix_Display_Control
* Description  : Function to control dynamic lighting of the matrix
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Matrix_Display_Control(void)
{
	P2 = matrix_digit_table[matrix_digit];			//Matrix Row Control

	P1 = matrix_data1[matrix_digit];			//Matrix column control
	P2 = P2 | matrix_data1_2[matrix_digit];
	P7 = matrix_data2[matrix_digit];
	P0 = matrix_data3[matrix_digit];

	matrix_digit++;
	if(matrix_digit == 5)
	{
		matrix_digit = 0;
	}
}

/***********************************************************************************************************************
* Function Name: Matrix_Display_Update
* Description  : Function to update the matrix display
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Matrix_Display_Update(void)
{
	int i,j,k;
	int l = 4;

	for(i = 1; i < 6; i++)						//Update the display of the first matrix
	{
		for(j = 7; j > 0; j--)
		{
			if(j == 3)
			{
				k <<= 3;
				if(screen[i][j] == 1)
				{
					matrix_data1_2[l] = 0x80;
				}
				else
				{
					matrix_data1_2[l] = 0;
				}
			}
			else if(j == 2)
			{
				if(screen[i][j] == 1)
				{
					k = k | 0x80;
				}
				else
				{
					k = k & 0x7F;
				}
			}
			else if(j == 1)
			{
				if(screen[i][j] == 1)
				{
					k = k | 0x01;
				}
				else
				{
					k = k & 0xFE;
				}
			}
			else
			{
				k <<= 1;
				k = k | screen[i][j];
			}
		}
		matrix_data1[l] = k;
		l--;
	}
	k = 0;
	l = 4;

	for(i = 6; i < 11; i++)						//Update the display of the second matrix
	{
		for(j = 7; j > 0; j--)
		{
			k <<= 1;
			k = k | screen[i][j];
		}
		matrix_data2[l] = k;
		l--;
	}
	k = 0;
	l = 4;

	for(i = 11; i < 16; i++)					//Update the display of the third matrix
	{
		for(j = 7; j > 0; j--)
		{
			k <<= 1;
			k = k | screen[i][j];
		}
		matrix_data3[l] = k;
		l--;
	}
}
